@@ -1,7 +1,7 @@
 /*
  * -----------------------------------------------------------------
- * $Revision: 1.22.2.1 $
- * $Date: 2005/01/24 21:46:02 $
+ * $Revision: 1.37 $
+ * $Date: 2006/03/24 02:37:58 $
  * -----------------------------------------------------------------
  * Programmer(s): Allan Taylor, Alan Hindmarsh, Radu Serban, and
  *                Aaron Collier @ LLNL
@@ -22,8 +22,40 @@
 extern "C" {
 #endif
 
-#include "nvector.h"
-#include "sundialstypes.h"
+#include <stdio.h>
+
+#include "sundials_nvector.h"
+
+/*
+ * -----------------------------------------------------------------
+ * KINSOL return flags
+ * -----------------------------------------------------------------
+ */
+
+#define KIN_SUCCESS             0
+#define KIN_INITIAL_GUESS_OK    1
+#define KIN_STEP_LT_STPTOL      2
+
+#define KIN_WARNING             99
+
+#define KIN_MEM_NULL            -1
+#define KIN_ILL_INPUT           -2
+#define KIN_NO_MALLOC           -3
+#define KIN_MEM_FAIL            -4
+#define KIN_LINESEARCH_NONCONV  -5
+#define KIN_MAXITER_REACHED     -6
+#define KIN_MXNEWT_5X_EXCEEDED  -7
+#define KIN_LINESEARCH_BCFAIL   -8
+#define KIN_LINSOLV_NO_RECOVERY -9
+#define KIN_LINIT_FAIL          -10
+#define KIN_LSETUP_FAIL         -11
+#define KIN_LSOLVE_FAIL         -12
+
+#define KIN_SYSFUNC_FAIL        -13
+#define KIN_FIRST_SYSFUNC_ERR   -14
+#define KIN_REPTD_SYSFUNC_ERR   -15
+
+
 
 /*
  * -----------------------------------------------------------------
@@ -85,12 +117,12 @@ extern "C" {
  * -----------------------------------------------------------------
  * Enumeration for global strategy
  * -----------------------------------------------------------------
- * Choices are KIN_INEXACT_NEWTON and KIN_LINESEARCH.
+ * Choices are KIN_NONE and KIN_LINESEARCH.
  * -----------------------------------------------------------------
  */
 
-#define KIN_INEXACT_NEWTON 1
-#define KIN_LINESEARCH     2
+#define KIN_NONE       0
+#define KIN_LINESEARCH 1
 
 /*
  * -----------------------------------------------------------------
@@ -104,12 +136,53 @@ extern "C" {
  * pointer.
  * 
  * Note: The user-defined routine (internally referenced by a
- * a pointer (type KINSysFn) named func) should have a void return
- * value type.
+ * a pointer (type KINSysFn) named func) should have an 'int' return
+ * value type. However, the return value is currently ignored.
  * -----------------------------------------------------------------
  */
 
-typedef void (*KINSysFn)(N_Vector uu, N_Vector fval, void *f_data );
+typedef int (*KINSysFn)(N_Vector uu, N_Vector fval, void *f_data );
+
+
+/*
+ * -----------------------------------------------------------------
+ * Type : KINErrHandlerFn
+ * -----------------------------------------------------------------
+ * A function eh, which handles error messages, must have type
+ * KINErrHandlerFn.
+ * The function eh takes as input the error code, the name of the
+ * module reporting the error, the error message, and a pointer to
+ * user data, the same as that passed to KINSetErrHandlerFn.
+ * 
+ * All error codes are negative, except KIN_WARNING which indicates 
+ * a warning (the solver continues).
+ *
+ * A KINErrHandlerFn has no return value.
+ * -----------------------------------------------------------------
+ */
+
+typedef void (*KINErrHandlerFn)(int error_code, 
+                               const char *module, const char *function, 
+                               char *msg, void *eh_data); 
+
+
+/*
+ * -----------------------------------------------------------------
+ * Type : KINInfoHandlerFn
+ * -----------------------------------------------------------------
+ * A function ih, which handles info messages, must have type
+ * KINInfoHandlerFn.
+ * The function ih takes as input the name of the module and of the
+ * function reporting the info message and a pointer to
+ * user data, the same as that passed to KINSetInfoHandlerFn.
+ * 
+ * A KINInfoHandlerFn has no return value.
+ * -----------------------------------------------------------------
+ */
+
+typedef void (*KINInfoHandlerFn)(const char *module, const char *function, 
+                                 char *msg, void *ih_data); 
+
 
 /*
  * -----------------------------------------------------------------
@@ -138,175 +211,213 @@ void *KINCreate(void);
  * -----------------------------------------------------------------
  * The following functions can be called to set optional inputs:
  *
- *     Function Name    |      Optional Input  [Default Value]
- *                      |
+ *     Function Name      |    Optional Input  [Default Value]
+ *                        |
  * -----------------------------------------------------------------
- *                      |
- * KINSetFdata          | pointer to user-allocated memory that is
- *                      | passed to the user-supplied subroutine
- *                      | implementing the nonlinear system function
- *                      | F(u)
- *                      | [NULL]
- *                      |
- * KINSetErrFile        | pointer (type FILE) indicating where all
- *                      | warning/error messages should be sent
- *                      | [stderr]
- *                      |
- * KINSetInfoFile       | pointer (type FILE) specifying where
- *                      | informative (non-error) messages should
- *                      | be sent (see KINSetPrintLevel)
- *                      | [stdout]
- *                      |
- * KINSetPrintLevel     | level of verbosity of output:
- *                      |
- *                      |  0  no statistical information is
- *                      |     displayed (default level)
- *                      |
- *                      |  1  for each nonlinear iteration display
- *                      |     the following information: the scaled
- *                      |     norm (L2) of the system function
- *                      |     evaluated at the current iterate, the
- *                      |     scaled norm of the Newton step (only if
- *                      |     using KIN_INEXACT_NEWTON), and the
- *                      |     number of function evaluations performed
- *                      |     thus far
- *                      |
- *                      |  2  display level 1 output and the
- *                      |     following values for each iteration:
- *                      |
- *                      |       fnorm (L2) = ||fscale*func(u)||_L2
- *                      |       (only for KIN_INEXACT_NEWTON)
- *                      |
- *                      |       scaled fnorm (for stopping) =
- *                      |       ||fscale*ABS(func(u))||_L-infinity
- *                      |       (for KIN_INEXACT_NEWTON and
- *                      |       KIN_LINESEARCH)
- *                      |
- *                      |  3  display level 2 output plus additional
- *                      |     values used by the global strategy
- *                      |     (only if using KIN_LINESEARCH), and
- *                      |     statistical information for the linear
- *                      |     solver
- *                      | [0]
- *                      |
- * KINSetNumMaxIters    | maximum number of nonlinear iterations
- *                      | [MXITER_DEFAULT] (defined in kinsol.c)
- *                      |
- * KINSetNoPrecInit     | flag controlling whether or not the
- *                      | KINSol routine makes an initial call
- *                      | to the preconditioner setup routine (pset)
- *                      | (possible values are TRUE and FALSE)
- *                      | [FALSE]
- *                      |
- * KINSetMaxPrecCalls   | maximum number of nonlinear iterations
- *                      | that may be performed between calls to
- *                      | the preconditioner setup routine (pset)
- *                      | [MSBPRE] (defined in kinsol.c)
- *                      |
- * KINSetEtaForm        | flag indicating which method to use to
- *                      | compute the value of the eta coefficient
- *                      | used in the calculation of the linear
- *                      | solver convergence tolerance:
- *                      |
- *                      |  eps = (eta+uround)*||fscale*func(u)||_L2
- *                      |
- *                      | the linear solver tests for convergence by
- *                      | checking if the following inequality has
- *                      | been satisfied:
- *                      |
- *                      |  ||fscale*(func(u)+J(u)*p)||_L2 <= eps
- *                      |
- *                      | where J(u) is the system Jacobian
- *                      | evaluated at the current iterate, and p
- *                      | denotes the Newton step
- *                      |
- *                      | choices for computing eta are as follows:
- *                      |
- *                      |  KIN_ETACHOICE1  (refer to KINForcingTerm)
- *                      |
- *                      |  eta = ABS(||F(u_k+1)||_L2-||F(u_k)+J(u_k)*p_k||_L2)
- *                      |        ---------------------------------------------
- *                      |                        ||F(u_k)||_L2
- *                      | 
- *                      |  KIN_ETACHOICE2  (refer to KINForcingTerm)
- *                      |
- *                      |                [ ||F(u_k+1)||_L2 ]^alpha
- *                      |  eta = gamma * [ --------------- ]
- *                      |                [  ||F(u_k)||_L2  ]
- *                      |
- *                      |  where gamma = [0,1] and alpha = (1,2]
- *                      |
- *                      |  KIN_ETACONSTANT  use a constant value for eta
- *                      | [KIN_ETACHOICE1]
- *                      |
- * KINSetEtaConstValue  | constant value of eta - use with
- *                      | KIN_ETACONSTANT option
- *                      | [0.1]
- *                      |
- * KINSetEtaParams      | values of eta_gamma (egamma) and eta_alpha
- *                      | (ealpha) coefficients - use with KIN_ETACHOICE2
- *                      | option
- *                      | [0.9 and 2.0]
- *                      |
- * KINSetNoMinEps       | flag controlling whether or not the value
- *                      | of eps is bounded below by 0.01*fnormtol
- *                      | (see KINSetFuncNormTol)
- *                      |
- *                      |  FALSE  constrain value of eps by setting
- *                      |         to the following:
- *                      |
- *                      |          eps = MAX{0.01*fnormtol, eps}
- *                      |
- *                      |  TRUE  do not constrain value of eps
- *                      | [FALSE]
- *                      |
- * KINSetMaxNewtonStep  | maximum scaled length of Newton step
- *                      | (reset to value of one if user-supplied
- *                      | value is less than one)
- *                      | [1000*||uscale*u_0||_L2]
- *                      |
- * KINSetRelErrFunc     | real scalar equal to realative error in
- *                      | computing F(u) (used in difference-
- *                      | quotient approximation of matrix-vector
- *                      | product J(u)*v)
- *                      | [(uround)^1/2]
- *                      |
- * KINSetFuncNormTol    | real scalar used as stopping tolerance on
- *                      | ||fscale*ABS(func(u))||_L-infinity (see
- *                      | KINStop and KINInitialStop)
- *                      | [(uround)^1/3]
- *                      |
- * KINSetScaledStepTol  | real scalar used as stopping tolerance on
- *                      | the maximum scaled step length:
- *                      |
- *                      |  ||    u_k+1 - u_k    ||
- *                      |  || ----------------- ||_L-infinity
- *                      |  || ABS(u_k+1)+uscale ||
- *                      |
- *                      | (see KINStop)
- *                      | [(uround)^2/3]
- *                      |
- * KINSetConstraints    | pointer to an array (type N_Vector) of
- *                      | constraints on the solution vector u
- *                      | 
- *                      | if constraints[i] =
- *                      |
- *                      |   0  u[i] not constrained
- *                      |
- *                      |  +1  u[i] constrained to be >= 0
- *                      |  -1  u[i] constrained to be <= 0
- *                      |
- *                      |  +2  u[i] constrained to be > 0
- *                      |  -2  u[i] constrained to be < 0
- *                      |
- *                      | if a NULL pointer is given, then no
- *                      | constraints are applied to vector u
- *                      | [NULL]
- *                      |
- * KINSetSysFunc        | set the user-provided routine which
- *                      | defines the nonlinear problem to be 
- *                      | solved
- *                      | [none]
+ *                        |
+ * KINSetErrHandlerFn     | user-provided ErrHandler function.
+ *                        | [internal]
+ *                        |
+ * KINSetErrFile          | pointer (type FILE) indicating where all
+ *                        | warning/error messages should be sent
+ *                        | if the default internal error handler 
+ *                        | is used
+ *                        | [stderr]
+ *                        |
+ * KINSetPrintLevel       | level of verbosity of output:
+ *                        |
+ *                        |  0  no statistical information is
+ *                        |     displayed (default level)
+ *                        |
+ *                        |  1  for each nonlinear iteration display
+ *                        |     the following information: the scaled
+ *                        |     norm (L2) of the system function
+ *                        |     evaluated at the current iterate, the
+ *                        |     scaled norm of the Newton step (only if
+ *                        |     using KIN_NONE), and the
+ *                        |     number of function evaluations performed
+ *                        |     thus far
+ *                        |
+ *                        |  2  display level 1 output and the
+ *                        |     following values for each iteration:
+ *                        |
+ *                        |       fnorm (L2) = ||fscale*func(u)||_L2
+ *                        |       (only for KIN_NONE)
+ *                        |
+ *                        |       scaled fnorm (for stopping) =
+ *                        |       ||fscale*ABS(func(u))||_L-infinity
+ *                        |       (for KIN_NONE and
+ *                        |       KIN_LINESEARCH)
+ *                        |
+ *                        |  3  display level 2 output plus additional
+ *                        |     values used by the global strategy
+ *                        |     (only if using KIN_LINESEARCH), and
+ *                        |     statistical information for the linear
+ *                        |     solver
+ *                        | [0]
+ *                        |
+ * KINSetInfoHandlerFn    | user-provided InfoHandler function.
+ *                        | [internal]
+ *                        |
+ * KINSetInfoFile         | pointer (type FILE) specifying where
+ *                        | informative (non-error) messages should
+ *                        | be sent if the default internal info
+ *                        | handler is used
+ *                        | [stdout]
+ *                        |
+ * KINSetFdata            | pointer to user-allocated memory that is
+ *                        | passed to the user-supplied subroutine
+ *                        | implementing the nonlinear system function
+ *                        | F(u)
+ *                        | [NULL]
+ *                        |
+ * KINSetNumMaxIters      | maximum number of nonlinear iterations
+ *                        | [MXITER_DEFAULT] (defined in kinsol_impl.h)
+ *                        |
+ * KINSetNoInitSetup      | flag controlling whether or not the
+ *                        | KINSol routine makes an initial call
+ *                        | to the linear solver setup routine (lsetup)
+ *                        | (possible values are TRUE and FALSE)
+ *                        | [FALSE]
+ *                        |
+ * KINSetNoResMon         | flag controlling whether or not the nonlinear
+ *                        | residual monitoring scheme is used to control
+ *                        | Jacobian updating (possible values are TRUE
+ *                        | and FALSE)
+ *                        | [FALSE if using direct linear solver]
+ *                        | [TRUE if using inexact linear solver]
+ *                        |
+ * KINSetMaxSetupCalls    | mbset, number of nonlinear iteraions, such 
+ *                        | that a call to the linear solver setup routine
+ *                        | (lsetup) is forced every mbset iterations.
+ *                        | If mbset=1, lsetup s called at every iteration.
+ *                        | [MSBSET_DEFAULT] (defined in kinsol_impl.h)
+ *                        |
+ * KINSetMaxSubSetupCalls | mbsetsub is the number of nonlinear iterations
+ *                        | between checks by the nonlinear residual
+ *                        | monitoring algorithm (specifies length of
+ *                        | subinterval)
+ *                        | NOTE: mbset should be a multiple of mbsetsub
+ *                        | [MSBSET_SUB_DEFAULT] (defined in kinsol_impl.h)
+ *                        |
+ * KINSetEtaForm          | flag indicating which method to use to
+ *                        | compute the value of the eta coefficient
+ *                        | used in the calculation of the linear
+ *                        | solver convergence tolerance:
+ *                        |
+ *                        |  eps = (eta+uround)*||fscale*func(u)||_L2
+ *                        |
+ *                        | the linear solver tests for convergence by
+ *                        | checking if the following inequality has
+ *                        | been satisfied:
+ *                        |
+ *                        |  ||fscale*(func(u)+J(u)*p)||_L2 <= eps
+ *                        |
+ *                        | where J(u) is the system Jacobian
+ *                        | evaluated at the current iterate, and p
+ *                        | denotes the Newton step
+ *                        |
+ *                        | choices for computing eta are as follows:
+ *                        |
+ *                        |  KIN_ETACHOICE1  (refer to KINForcingTerm)
+ *                        |
+ *                        |  eta = ABS(||F(u_k+1)||_L2-||F(u_k)+J(u_k)*p_k||_L2)
+ *                        |        ---------------------------------------------
+ *                        |                        ||F(u_k)||_L2
+ *                        | 
+ *                        |  KIN_ETACHOICE2  (refer to KINForcingTerm)
+ *                        |
+ *                        |                [ ||F(u_k+1)||_L2 ]^alpha
+ *                        |  eta = gamma * [ --------------- ]
+ *                        |                [  ||F(u_k)||_L2  ]
+ *                        |
+ *                        |  where gamma = [0,1] and alpha = (1,2]
+ *                        |
+ *                        |  KIN_ETACONSTANT  use a constant value for eta
+ *                        | [KIN_ETACHOICE1]
+ *                        |
+ * KINSetEtaConstValue    | constant value of eta - use with
+ *                        | KIN_ETACONSTANT option
+ *                        | [0.1]
+ *                        |
+ * KINSetEtaParams        | values of eta_gamma (egamma) and eta_alpha
+ *                        | (ealpha) coefficients - use with KIN_ETACHOICE2
+ *                        | option
+ *                        | [0.9 and 2.0]
+ *                        |
+ * KINSetResMonParams     | values of omega_min and omega_max scalars
+ *                        | used by nonlinear residual monitoring
+ *                        | algorithm (see KINStop)
+ *                        | [0.00001 and 0.9]
+ *                        |
+ * KINSetResMonConstValue | constant value used by residual monitoring
+ *                        | algorithm. If omega=0, then it is estimated
+ *                        | using omega_min and omega_max.
+ *                        | [0.0]
+ *                        |
+ * KINSetNoMinEps         | flag controlling whether or not the value
+ *                        | of eps is bounded below by 0.01*fnormtol
+ *                        | (see KINSetFuncNormTol)
+ *                        |
+ *                        |  FALSE  constrain value of eps by setting
+ *                        |         to the following:
+ *                        |
+ *                        |          eps = MAX{0.01*fnormtol, eps}
+ *                        |
+ *                        |  TRUE  do not constrain value of eps
+ *                        | [FALSE]
+ *                        |
+ * KINSetMaxNewtonStep    | maximum scaled length of Newton step
+ *                        | (reset to value of one if user-supplied
+ *                        | value is less than one)
+ *                        | [1000*||uscale*u_0||_L2]
+ *                        |
+ * KINSetMaxBetaFails     | maximum number of beta condition failures
+ *                        | in the line search algorithm.
+ *                        | [MXNBCF_DEFAULT] (defined in kinsol_impl.h)
+ *                        |
+ * KINSetRelErrFunc       | real scalar equal to realative error in
+ *                        | computing F(u) (used in difference-
+ *                        | quotient approximation of matrix-vector
+ *                        | product J(u)*v)
+ *                        | [(uround)^1/2]
+ *                        |
+ * KINSetFuncNormTol      | real scalar used as stopping tolerance on
+ *                        | ||fscale*ABS(func(u))||_L-infinity (see
+ *                        | KINStop and KINInitialStop)
+ *                        | [(uround)^1/3]
+ *                        |
+ * KINSetScaledStepTol    | real scalar used as stopping tolerance on
+ *                        | the maximum scaled step length:
+ *                        |
+ *                        |  ||    u_k+1 - u_k    ||
+ *                        |  || ----------------- ||_L-infinity
+ *                        |  || ABS(u_k+1)+uscale ||
+ *                        |
+ *                        | (see KINStop)
+ *                        | [(uround)^2/3]
+ *                        |
+ * KINSetConstraints      | pointer to an array (type N_Vector) of
+ *                        | constraints on the solution vector u
+ *                        | 
+ *                        | if constraints[i] =
+ *                        |
+ *                        |   0  u[i] not constrained
+ *                        |
+ *                        |  +1  u[i] constrained to be >= 0
+ *                        |  -1  u[i] constrained to be <= 0
+ *                        |
+ *                        |  +2  u[i] constrained to be > 0
+ *                        |  -2  u[i] constrained to be < 0
+ *                        |
+ *                        | if a NULL pointer is given, then no
+ *                        | constraints are applied to vector u
+ *                        | [NULL]
+ *                        |
+ * KINSetSysFunc          | set the user-provided routine which
+ *                        | defines the nonlinear problem to be 
+ *                        | solved
+ *                        | [none]
  * -----------------------------------------------------------------
  * The possible return values for the KINSet* subroutines are the
  * following:
@@ -327,18 +438,25 @@ void *KINCreate(void);
  * -----------------------------------------------------------------
  */
 
-int KINSetFdata(void *kinmem, void *f_data);
+int KINSetErrHandlerFn(void *kinmem, KINErrHandlerFn ehfun, void *eh_data);
 int KINSetErrFile(void *kinmem, FILE *errfp);
+int KINSetInfoHandlerFn(void *kinmem, KINInfoHandlerFn ihfun, void *ih_data);
 int KINSetInfoFile(void *kinmem, FILE *infofp);
+int KINSetFdata(void *kinmem, void *f_data);
 int KINSetPrintLevel(void *kinmemm, int printfl);
 int KINSetNumMaxIters(void *kinmem, long int mxiter);
-int KINSetNoPrecInit(void *kinmem, booleantype noPrecInit);
-int KINSetMaxPrecCalls(void *kinmem, long int msbpre);
+int KINSetNoInitSetup(void *kinmem, booleantype noInitSetup);
+int KINSetNoResMon(void *kinmem, booleantype noNNIResMon);
+int KINSetMaxSetupCalls(void *kinmem, long int msbset);
+int KINSetMaxSubSetupCalls(void *kinmem, long int msbsetsub);
 int KINSetEtaForm(void *kinmem, int etachoice);
 int KINSetEtaConstValue(void *kinmem, realtype eta);
 int KINSetEtaParams(void *kinmem, realtype egamma, realtype ealpha);
+int KINSetResMonParams(void *kinmem, realtype omegamin, realtype omegamax);
+int KINSetResMonConstValue(void *kinmem, realtype omegaconst);
 int KINSetNoMinEps(void *kinmem, booleantype noMinEps);
 int KINSetMaxNewtonStep(void *kinmem, realtype mxnewtstep);
+int KINSetMaxBetaFails(void *kinmem, long int mxnbcf);
 int KINSetRelErrFunc(void *kinmem, realtype relfunc);
 int KINSetFuncNormTol(void *kinmem, realtype fnormtol);
 int KINSetScaledStepTol(void *kinmem, realtype scsteptol);
@@ -421,7 +539,7 @@ int KINMalloc(void *kinmem, KINSysFn func, N_Vector tmpl);
  *      the nonlinear system F(u) = 0
  *
  *  strategy  global strategy applied to Newton step if unsatisfactory
- *            (KIN_INEXACT_NEWTON or KIN_LINESEARCH)
+ *            (KIN_NONE or KIN_LINESEARCH)
  *
  *  u_scale  vector containing diagonal elements of scaling matrix
  *           for vector u chosen so that the components of
@@ -590,6 +708,9 @@ int KINSol(void *kinmem, N_Vector uu, int strategy,
  *                           |
  *                           |  ||uscale*p||_L2
  *                           |
+ * KINGetReturnFlagName      | returns the name of the constant
+ *                           | associated with a KINSOL return flag
+ *                           |
  * -----------------------------------------------------------------
  *
  * The possible return values for the KINSet* subroutines are the
@@ -610,6 +731,7 @@ int KINGetNumBetaCondFails(void *kinmem, long int *nbcfails);
 int KINGetNumBacktrackOps(void *kinmem, long int *nbacktr);
 int KINGetFuncNorm(void *kinmem, realtype *fnorm);
 int KINGetStepLength(void *kinmem, realtype *steplength);
+char *KINGetReturnFlagName(int flag);
 
 /*
  * -----------------------------------------------------------------
@@ -623,31 +745,7 @@ int KINGetStepLength(void *kinmem, realtype *steplength);
  * -----------------------------------------------------------------
  */
 
-void KINFree(void *kinmem);
-
-/*
- * -----------------------------------------------------------------
- * KINSOL return flags
- * -----------------------------------------------------------------
- */
-
-#define KIN_SUCCESS          0
-#define KIN_INITIAL_GUESS_OK 1
-#define KIN_STEP_LT_STPTOL   2
-
-#define KIN_MEM_NULL            -1
-#define KIN_ILL_INPUT           -2
-#define KIN_NO_MALLOC           -3
-#define KIN_MEM_FAIL            -4
-#define KIN_LINESEARCH_NONCONV  -5
-#define KIN_MAXITER_REACHED     -6
-#define KIN_MXNEWT_5X_EXCEEDED  -7
-#define KIN_LINESEARCH_BCFAIL   -8
-#define KIN_LINSOLV_NO_RECOVERY -9
-#define KIN_LINIT_FAIL          -10
-#define KIN_LSETUP_FAIL         -11
-#define KIN_LSOLVE_FAIL         -12
-#define KIN_PDATA_NULL          -13
+void KINFree(void **kinmem);
 
 /*
  * -----------------------------------------------------------------
@@ -668,8 +766,7 @@ void KINFree(void *kinmem);
  * If the necessary variables have been successfully initialized,
  * then the kin_linit function should return 0 (zero). Otherwise,
  * the subroutine should indicate a failure has occurred by
- * returning a negative integer value and printing an informative
- * error message to the kin_mem->kin_errfp pointer.
+ * returning a non-zero integer value.
  * -----------------------------------------------------------------
  */
 
@@ -679,17 +776,15 @@ void KINFree(void *kinmem);
  * -----------------------------------------------------------------
  * kin_lsetup interfaces with the user-supplied pset subroutine (the
  * preconditioner setup routine), and updates relevant variable
- * values (see KINSpgmrSetup). Simply stated, the kin_lsetup routine
- * prepares the linear solver for a subsequent call to the
- * user-supplied kin_lsolve function.
+ * values (see KINSpgmrSetup/KINSpbcgSetup). Simply stated, the
+ * kin_lsetup routine prepares the linear solver for a subsequent
+ * call to the user-supplied kin_lsolve function.
  *
  *  kinmem  pointer to an internal memory block allocated during
  *          prior calls to KINCreate and KINMalloc
  *
- * If successful, the kin_lsetup routine should return 0 (zero). If
- * a recoverable error occurs, then the subroutine should return a
- * positive integer value. However, if an unrecoverable error
- * occurs, then the function should return a negative integer value.
+ * If successful, the kin_lsetup routine should return 0 (zero).
+ * Otherwise it should return a non-zero value.
  * -----------------------------------------------------------------
  */
 
@@ -703,7 +798,7 @@ void KINFree(void *kinmem);
  * and must increment the relevant counter variable values in
  * addition to computing certain values used by the global strategy
  * and forcing term routines (see KINInexactNewton, KINLineSearch,
- * KINForcingTerm, and KINSpgmrSolve).
+ * KINForcingTerm, and KINSpgmrSolve/KINSpbcgSolve).
  *
  *  kinmem  pointer to an internal memory block allocated during
  *          prior calls to KINCreate and KINMalloc
@@ -720,27 +815,23 @@ void KINFree(void *kinmem);
  *  res_norm  holds the value of the L2-norm (Euclidean norm) of
  *            the residual vector upon return
  *
- * If successful, the kin_lsolve routine should return 0 (zero). If
- * a recoverable error occurs, then the subroutine should return a
- * positive integer value. However, if an unrecoverable error
- * occurs, then the function should return a negative integer value.
+ * If successful, the kin_lsolve routine should return 0 (zero).
+ * Otherwise it should return a positive value if a re-evaluation
+ * of the lsetup function could recover, or a negative value if
+ * no such recovery is possible.
  * -----------------------------------------------------------------
  */
 
 /*
  * -----------------------------------------------------------------
- * Function : int (*kin_lfree)(KINMem kin_mem)
+ * Function : void (*kin_lfree)(KINMem kin_mem)
  * -----------------------------------------------------------------
  * kin_lfree is called by KINFree and should free (deallocate) all
  * system memory resources allocated for the linear solver module
- * (see KINSpgmrFree).
+ * (see KINSpgmrFree/KINSpbcgFree).
  *
  *  kinmem  pointer to an internal memory block allocated during
  *          prior calls to KINCreate and KINMalloc
- *
- * If successful, the kin_lfree routine should return 0 (zero). If
- * an error occurs, then the subroutine should return a negative
- * integer value.
  * -----------------------------------------------------------------
  */
 
